'use strict'

const EventEmitter = require('events')

// Subclass of EventEmitter that adds 3 new methods:
// * addRevocableListener() (analog of addListener())
// * revocableOn()          (analog of on())
// * revocableOnce()        (analog of once())
//
// The only difference with these methods and their analogs is that the return
// value is a function that will remove the listener, instead of returning the
// eventEmitter instance.
//
// eg:
//   const revokeListener = thing.revocableOn('foo', () => doThing())
//   ...
//   revokeListener() // stop listening

class RevocableEventEmitter extends EventEmitter {
  addRevocableListener (eventName, listener) {
    this.addListener(eventName, listener)

    const self = this

    return function revokeListener () {
      self.removeListener(eventName, listener)
    }
  }

  revocableOn (eventName, listener) {
    return this.addRevocableListener(eventName, listener)
  }

  revocableOnce (eventName, listener) {
    this.once(eventName, listener)

    const self = this

    return function revokeListener () {
      self.removeListener(eventName, listener)
    }
  }
}

module.exports = RevocableEventEmitter
