'use strict'

exports.launch = launch
const zmqRelay = require('../zmqrelay')
const util = require('../util')
const Logger = util.getLogger(__filename)

function launch (Context, Config) {
  return new Promise((resolve, reject) => {
    Logger.info('launching in Relay mode')
    zmqRelay.launch(Context, (err, relay) => {
      if (err) return reject(err)

      Logger.info(`Listening to SIGHUP for config reloading on PID ${process.pid}`)

      process.on('SIGHUP', () => {
        setImmediate(() => {
          let conf
          try {
            conf = Config.getConfig()
          } catch (err) {
            Logger.log(`Config reload failed: ${err.message}`)
            return
          }
          if (!conf) {
            Logger.log('Config reload failed, invalid configuration.')
            return
          }

          Context.config = conf
          relay.reloadConfig(Context.config)
        })
      })

      Logger.info('launched ZeroMQ relay')
      Context.zmqRelay = relay
      resolve()
    })
  })
}
