'use strict'

const RevocableEventEmitter = require('./RevocableEventEmitter')

const util = require('./util')

const Logger = util.getLogger(__dirname, __filename)

exports.create = create

function create (context) {
  return new EventManager(context)
}

// Manages system events.
class EventManager extends RevocableEventEmitter {
  constructor (context) {
    super()
    this._context = context
    Logger.debug('created EventManager')
  }

  // emit an event with the specified name and args
  // will also be emitted with the name "system-event" passing the name/args
  emitSystemEvent (name, agentId, args, time) {
    const systemEvent = this.createEvent(name, agentId, args, time)

    // emit as a "system-event"
    this.emit('system-event', systemEvent)

    if (this._context.influxdbWriter) {
      this._context.influxdbWriter.writeEvent(name, agentId, args, time)
    }

    // emit as the named event also
    this.emit(name, systemEvent)
  }

  createEvent (name, agentId, args, time) {
    if (time == null) time = new Date().toISOString()

    if (args == null) {
      Logger.warn(`emitSystemEvent (${name}) called with no content.`)
    }

    // virtually remove agent from output, if null
    if (agentId == null) agentId = undefined

    return {
      time: time,
      event: name,
      agent: agentId,
      args: args
    }
  }
}
