'use strict'

// TODO: move to integration tests, and update comment in fields.js

const FieldManager = require('./field-manager')
const fieldManager = FieldManager.create()
const remainingFields = fieldManager.fields

const nsolid = require('nsolid')
nsolid.metrics(processMetrics)

function processMetrics (err, metrics) {
  if (err) {
    console.log('error getting metrics: ', err)
    process.exit(1)
  }

  const object = FieldManager.flattenObject(metrics)
  for (let name in object) {
    const field = remainingFields[name]
    if (field == null) {
      const type = getType(object[name])
      console.log(`missing: metrics ${name} ${type}`)
      continue
    }

    validate(field, 'metrics', object[name])
    delete remainingFields[name]
  }

  nsolid.info(processInfo)
}

function processInfo (err, info) {
  if (err) {
    console.log('error getting info: ', err)
    process.exit(1)
  }

  const object = FieldManager.flattenObject(info)
  for (let name in object) {
    const field = remainingFields[name]
    if (field == null) {
      const type = getType(object[name])
      console.log(`missing: info ${name} ${type}`)
      continue
    }

    validate(field, 'info', object[name])
    delete remainingFields[name]
  }

  if (Object.keys(remainingFields).length !== 0) {
    console.log('fields defined but not found in live data')
  }

  for (let name in remainingFields) {
    console.log(`  ${name}`)
  }

  console.log('if nothing printed, then things are A-OK!')
}

function validate (field, domain, value) {
  if (field.domain !== domain) {
    console.log(`for field ${field.name} got domain ${domain} instead of ${field.domain}`)
  }

  if (['number', 'string', 'array'].indexOf(field.type) === -1) {
    console.log(`bad type for field ${field.name}: ${field.type}`)
  }

  if (field.type === 'string') {
    if (typeof value !== 'string' && value != null) {
      console.log(`unexpected type value for field ${field.name}: ${field.type}: ${JSON.stringify(value)}`)
    }
  }

  if (field.type === 'number') {
    if (typeof value !== 'number') {
      console.log(`unexpected type value for field ${field.name}: ${field.type}: ${JSON.stringify(value)}`)
    }
  }

  if (field.type === 'array') {
    if (!Array.isArray(value)) {
      console.log(`unexpected type value for field ${field.name}: ${field.type}: ${JSON.stringify(value)}`)
    }
  }
}

function getType (object) {
  const type = typeof object
  if (type === 'number') return 'number'
  if (type === 'string') return 'string'
  if (Array.isArray(object)) return 'array'
  return 'string'
}
