'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const httpUtils = require('../http-utils')
const Logger = util.getLogger(__dirname, __filename)
const async = require('async')
const KEY = 'integrations'

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context

  Logger.debug('handling request')

  const id = req.query && req.query.id
  if (id == null) {
    const err = `no id provided`
    Logger.warn(err)
    res.status(400)
    return res.send({ error: { message: err } })
  }

  validate(context, id, validationError => {
    if (validationError != null) {
      Logger.warn(validationError)
      res.status(400)
      return res.send({ error: { message: validationError } })
    }

    // update the settings with settingsManager
    httpUtils.deleteSettingByType(req, res, KEY, (err) => {
      if (err) {
        Logger.error(err, 'error updating setttings')
      }
    })
  })
}

function validate (context, id, cb) {
  async.parallel({
    notifs: acb => context.settingsManager.getSettings('notifications', acb),
    views: acb => context.settingsManager.getSettings('savedViews', acb)
  }, (err, res) => {
    if (err) return cb(err.message)
    const notifications = Object.keys(res.notifs).map(key => res.notifs[key])
    const savedViews = Object.keys(res.views).map(key => res.views[key])

    for (let key of notifications) {
      const notification = notifications[key]
      if (notification == null) continue

      const actions = notification.actions || []
      for (let action of actions) {
        if (action == null) continue
        if (action.integrationId === id) return cb(`integration in use by notification ${key}`)
      }
    }

    for (let savedView of savedViews) {
      if (savedView == null) continue

      const actions = savedView.actions || []
      for (let action of actions) {
        if (action == null) continue
        if (action.integrationId === id) return cb(`integration in use by notification ${savedView.name}`)
      }
    }

    cb()
  })
}
