'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

const nsolid = require('nsolid')

// Handle the HTTP request
function handleRequest (req, res) {
  nsolid.info((err, info) => {
    if (err != null) return processError(res, 'info', err)
    nsolid.metrics((err, metrics) => {
      if (err) {
        Logger.warn(`Couldn't get metrics for server but returning info anyway: ${err.message}`)
      }

      res.status(200)
      res.send({
        time: new Date(),
        info: info,
        metrics: metrics
      })
    })
  })
}

// for errors, print a warning and return a 500 response
function processError (res, type, err) {
  Logger.warn(err, `error processing ${type} request`)
  res.status(500)
  res.send({error: {message: 'error processing request'}})
}
