'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const params = {
    company: req.body.company,
    email: req.body.email,
    marketingOptIn: req.body.marketingOptIn,
    lastname: req.body.lastName,
    name: req.body.firstName,
    provider: context.provider.type,
    providerData: req.body.providerData || '',
    state: 'ACTIVE',
    trial: req.body.trial
  }

  Logger.debug(`handleRequest: provider: ${context.provider.type}, req.body: ${JSON.stringify(req.body)}`)

  function responseHandler (action) {
    return function (err, license) {
      if (err) {
        res.status(400)
        res.send({ error: { message: `Could not ${action} license.` } })
        Logger.error(err, `Could not ${action} license.`)
        return
      }

      res.status(200)
      res.send(context.provider.license)
    }
  }

  if (params.email == null || params.email === '') {
    return responseHandler('create')(new Error('Email is required to create a license.'))
  }

  return context.provider.createLicense(params, responseHandler('create'))
}
