'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  Logger.debug(`handleRequest(${JSON.stringify(req.body)})`)
  const context = req.nsConsole.context
  let offlineMode
  if (req.body.offlineMode === true || req.body.offlineMode === 'true') offlineMode = true
  if (req.body.offlineMode === false || req.body.offlineMode === 'false') offlineMode = false

  function responseHandler (err) {
    if (err) {
      res.status(400)
      res.send({ error: { message: 'Could not update license.' } })
      Logger.error(err, 'Could not update license.')
      return
    }

    res.status(200)
    res.send(context.provider.license)
  }

  if (req.body.license && req.body.license.token) {
    // When setting a JWT, it is ok to set offlineMode since we can validate
    // this offline.  If you also try to set a license key in addition to the
    // token, if the key passed and the one in the token do not match, consider
    // it an error.
    if (req.body.license && req.body.license.key) {
      const parsed = context.provider.parseToken(req.body.license.token)
      if (parsed.licenseKey !== req.body.license.key) {
        return responseHandler(new Error('License key does not match license token.'))
      }
    }

    context.provider.setToken(req.body.license.token)

    if (offlineMode != null) {
      return context.provider.setOfflineMode(offlineMode, responseHandler)
    } else {
      responseHandler()
    }
  } else if (req.body.license && req.body.license.key) {
    // If setting the license key but not providing a token, we'll need to be
    // able to validate it against the license server, so offlineMode cannot be
    // true
    if (offlineMode) {
      return responseHandler(new Error('Cannot validate license while offline, disable offlineMode or pass a token'))
    }

    // syncLicense calls LicenseKey.fromBody which implicitly forces
    // offlineMode to false, so no need to act on that directly
    context.provider.syncLicense({ key: req.body.license.key, notify: true }, responseHandler)
  } else if (req.body.license && req.body.license.email) {
    // If syncing the license by email, we'll need to be able to validate it
    // against the license server, so offlineMode cannot be true
    if (offlineMode) {
      return responseHandler(new Error('Cannot validate license while offline, disable offlineMode or pass a token'))
    }

    // syncLicense calls LicenseKey.fromBody which implicitly forces
    // offlineMode to false, so no need to act on that directly
    context.provider.syncLicense({ email: req.body.license.email, notify: true }, responseHandler)
  } else if (offlineMode != null) {
    // Not changing the token or license key, just want to set the offlineMode
    // flag.  This is ok if the existing license key has a token and is
    // validated.
    if (offlineMode) {
      if (!(context.provider.license.token && context.provider.license.valid)) {
        return responseHandler(new Error('Cannot opt out of network features without a validated license or token'))
      }
    }
    return context.provider.setOfflineMode(offlineMode, responseHandler)
  } else {
    return responseHandler(new Error(`Invalid arguments: ${req.body}`))
  }
}
