'use strict'

const through2 = require('through2')
const splice = require('stream-splice')
const ndjson = require('ndjson')
const flatnest = require('flatnest')

module.exports.rawFormat = through2.ctor()
module.exports.Consolize = Consolize
module.exports.filterFields = filterFields

// Add a final \n if we're in a TTY and it doesn't have one already
function Consolize (output) {
  var final = null
  return through2(function _noop (chunk, encoding, callback) {
    final = chunk[chunk.length - 1]
    callback(null, chunk)
  }, function _addNewline (callback) {
    if (output.isTTY) {
      if (final !== null && final !== 10) { // 0x10 = \n
        // only add newline if in TTY & doesn't already end with one
        this.push('\n')
      }
    }
    callback()
  })
}

function filterFields (fieldString) {
  if (fieldString == null) {
    return through2.ctor()
  }

  const fields = fieldString.replace(/\s+/, '').split(',').map((val) => {
    return new RegExp(`\\b${val}\\b`)
  })

  const filter = through2.obj((record, encoding, callback) => {
    const flat = flatnest.flatten(record)
    const filtered = {}
    for (const key in flat) {
      for (const idx in fields) {
        if (key.match(fields[idx])) {
          filtered[key] = flat[key]
        }
      }
    }
    return callback(null, flatnest.nest(filtered))
  })
  return splice(ndjson.parse(), filter, ndjson.serialize())
}
